import os
import pathlib
import tkinter as tk
import tkinter.ttk as ttk
import tkinter.filedialog
from multiprocessing.pool import ThreadPool

import pygubu
from pygubu.builder.widgets.dialog import Dialog
from tkinter import DISABLED, LEFT

from controllers.ControllerMap import ControllerMap


class ControllerDialogLoader:

    def __init__(self, controller_map: ControllerMap, master=None):
        self.builder = pygubu.Builder()

        root_path = pathlib.Path(__file__).parent.parent
        self.builder.add_resource_path(root_path)
        self.builder.add_from_file(f'{root_path}/views/ViewWindowLoader.ui')

        self.uiDialog: pygubu.builder.widgets.dialog.Dialog = self.builder.get_object('uiDialog', master)

        self.uiButtonOpenOk: ttk.Button = self.builder.get_object('uiButtonOk')
        self.uiButtonCancel: ttk.Button = self.builder.get_object('uiButtonCancel')
        self.uiButtonOpenPLY: ttk.Button = self.builder.get_object('uiButtonOpenPLY')
        self.uiButtonOpenTXT: ttk.Button = self.builder.get_object('uiButtonOpenTXT')

        self.uiCheckboxTXT: ttk.Checkbutton = self.builder.get_object('uiCheckboxTXT')
        self.uiCheckboxPLY: ttk.Checkbutton = self.builder.get_object('uiCheckboxPLY')

        self.uiCheckbox45Deg: ttk.Checkbutton = self.builder.get_object('uiCheckbox45Deg')

        self.uiStatusLabel: ttk.Label = self.builder.get_object('uiStatusLabel')
        self.uiLabelPLY: ttk.Label = self.builder.get_object('uiLabelPLY')
        self.uiLabelTXT: ttk.Label = self.builder.get_object('uiLabelTXT')

        self.uiProgressBar: ttk.Progressbar = self.builder.get_object('uiProgressBar')

        self.uiProgressBar.configure(
            orient='horizontal', mode='indeterminate'
        )

        self.uiCheckboxTXT_Checked = tk.BooleanVar()
        self.uiCheckboxPLY_Checked = tk.BooleanVar()
        self.uiCheckbox45Deg_Checked = tk.BooleanVar()

        self.uiStatusLabel_Text = tk.StringVar(value='Choose PLY and TXT files..')
        self.uiLabelPLY_Text = tk.StringVar(value='Not selected..')
        self.uiLabelTXT_Text = tk.StringVar(value='Not selected..')

        self.uiStatusLabel.configure(
            textvariable=self.uiStatusLabel_Text
        )
        self.uiLabelTXT.configure(
            textvariable=self.uiLabelTXT_Text
        )
        self.uiLabelPLY.configure(
            textvariable=self.uiLabelPLY_Text
        )

        self.uiCheckbox45Deg.configure(
            variable=self.uiCheckbox45Deg_Checked
        )
        self.uiCheckboxTXT.configure(
            state=DISABLED,
            variable=self.uiCheckboxTXT_Checked
        )
        self.uiCheckboxPLY.configure(
            state=DISABLED,
            variable=self.uiCheckboxPLY_Checked
        )

        self.uiButtonOpenOk.configure(
            command=self.uiButtonOpenOk_Click
        )
        self.uiButtonCancel.configure(
            command=self.uiButtonCancel_Click
        )

        self.uiButtonOpenTXT.configure(
            command=self.uiButtonOpenTXT_Click
        )
        self.uiButtonOpenPLY.configure(
            command=self.uiButtonOpenPLY_Click
        )

        self.filepath_ply = None
        self.filepath_txt = None
        self.controller_map: ControllerMap = controller_map

        self.is_loading = False

    def show(self, parent):
        if parent is not None:
            self.uiDialog.parent = parent
        self.uiDialog.set_modal(True)
        self.uiDialog.show()
        self.uiDialog.run()

    def showMain(self):
        self.uiDialog.show()
        self.uiDialog.run()

    def uiButtonOpenOk_Click(self):
        # debug code
        hostname = os.uname().nodename
        if hostname == 'Evaldss-MacBook-Pro.local':
            self.filepath_txt = '/Users/evalds/Documents/alarm_pointcloud_reports_2021_Q4/materials/2021-11-02_PLY_dataset_1/TXT/poses_Nikhil_P_20_D_1_1_1.txt'
            self.filepath_ply = '/Users/evalds/Documents/alarm_pointcloud_reports_2021_Q4/materials/2021-11-02_PLY_dataset_1/PLY/master_Nikhil_P_20_D_1_1_1.ply'

        if not self.is_loading:
            if self.filepath_ply:
                use_45deg_lines = self.uiCheckbox45Deg_Checked.get()
                self.uiStatusLabel_Text.set('Processing map..')
                self.is_loading = True
                self.uiProgressBar.start()
                self.thread_pool = ThreadPool()
                self.thread_pool.apply_async(self.controller_map.load_map_from_ply, (self.filepath_ply, self.filepath_txt, use_45deg_lines), callback=self.on_main_callback)

    def on_main_callback(self, map_result):
        self.uiDialog.close()
        self.controller_map.plot()

    def uiButtonOpenTXT_Click(self):
        if not self.is_loading:
            self.filepath_txt = tk.filedialog.askopenfilename(
                title='Open a txt file',
                initialdir='/',
                filetypes=(("txt files", "*.txt"), ("All files", "*.*")))
            if self.filepath_txt:
                self.uiLabelTXT_Text.set(os.path.basename(self.filepath_txt))
                self.uiCheckboxTXT_Checked.set(True)

    def uiButtonOpenPLY_Click(self):
        if not self.is_loading:
            self.filepath_ply = tk.filedialog.askopenfilename(
                title='Open a PLY file',
                initialdir='/',
                filetypes=(("PLY files", "*.ply"), ("All files", "*.*")))
            if self.filepath_ply:
                self.uiLabelPLY_Text.set(os.path.basename(self.filepath_ply))
                self.uiCheckboxPLY_Checked.set(True)

    def uiButtonCancel_Click(self):
        self.uiDialog.close()
