class DialogFlightPlan extends React.Component {
    constructor(props) {
        super(props);

        this.state = {
            selected_step_template: false,
            isDialogWarning:false,
            message: ""
        }
    }

    onClose = () => {
        this.props.onClose();
    }

    onExecute = () => {
        if(this.props.isFlightPlanExecuting) {
            if(this.props.onStopFlightPlan) {
                this.props.onStopFlightPlan();
            }
        }
        else {
            //Get the last step
            let steps_list = this.props.project.flight_plan.steps;
            let last_step;
            if (steps_list.length) {
                last_step = steps_list[steps_list.length - 1];
            }
            // If the flight plan is empty
            if (!last_step) {
                this.setState({
                    isDialogWarning: true,
                    message: "Cannot execute an empty Flight plan!",
                })
            } else if (last_step.step_type !== "Land") {
                // If the step is first one then check if it is take off
                this.setState({
                    isDialogWarning: true,
                    message: "Final step must be Land!",
                })
            } else {
                this.props.project.flight_plan.current_loop_count = 0;
                this.props.project.flight_plan.current_delay_before_execution_sec = 0;

                eel.add_log("INFO", "Initiating execution of flight plan.");
                if (this.props.project) {
                    for (let step of this.props.project.flight_plan.steps) {
                        step.step_status = "Pending";
                    }
                }
                if (this.props.onExecutePlan) {
                    this.props.onExecutePlan();
                }
            }
        }
    }



    handleChangeName = (name, uuid) => {
        this.props.onChangeNameOpen(name, uuid);
    }

    handleOnChange(event, ui) {
        // console.log('DOM changed!', event, ui);
    }

    render = () => {
        const style = {
            width: 600,
            backgroundColor: "#f3f3f3",
            border: '2px solid #000',
            borderTop: 0,
            p: 4,
            borderRadius: "0 0 18px 18px",
        };

        const contain_style = {
            position: 'absolute',
            top: '50%',
            left: '50%',
            width: "auto",
            transform: 'translate(-50%, -50%)',
        }

        const title_bar_style = {
            fontSize: "102%",
            fontFamily: 'Space Grotesk',
            fontWeight: "600",
            letterSpacing: "0.5px",
            lineHeight: "25px",
            textAlign: "center",
            width: 600,
            backgroundColor: "#fff",
            border: '2px solid #000',
            borderBottom: 0,
            p: 1,
            borderRadius: "18px 18px 0 0",
        }

        const grid_container = {
            fontFamily: 'Space Grotesk',
            fontWeight: 600,
            fontSize: "115%",
            display: "grid",
            gridTemplateColumns: "106px 150px 150px 106px",
            justifyItems: "center",
            verticalAlign: "top",
            marginBottom: "10px",
        }

        let value_repeat_count = "1";
        if(this.props.project && this.props.project.flight_plan) {
            if(!this.props.project.flight_plan.repeat_count) {
                this.props.project.flight_plan.repeat_count = 1;
            }
            value_repeat_count = this.props.project.flight_plan.repeat_count.toString();
        }
        if(this.props.isFlightPlanExecuting) {
            if(!this.props.project.flight_plan.current_loop_count) {
                this.props.project.flight_plan.current_loop_count = 0;
            }
            value_repeat_count = `${this.props.project.flight_plan.current_loop_count} / ${value_repeat_count}`;
        }

        let value_delay_before_execution_sec = "0";
        if(this.props.project && this.props.project.flight_plan) {
            if(!this.props.project.flight_plan.delay_before_execution_sec) {
                this.props.project.flight_plan.delay_before_execution_sec = 0;
            }
            value_delay_before_execution_sec = this.props.project.flight_plan.delay_before_execution_sec.toString();
        }

        if(this.props.isFlightPlanExecuting) {
            if(!this.props.project.flight_plan.current_delay_before_execution_sec) {
                this.props.project.flight_plan.current_delay_before_execution_sec = 0;
            }
            value_delay_before_execution_sec = `${this.props.project.flight_plan.current_delay_before_execution_sec} / ${value_delay_before_execution_sec}`;
        }

        let step_templates = [];
        if(this.props.project && this.props.project.flight_plan) {
            step_templates.push({
                step_type: "TakeOff",
                name: "Take Off",
                poi_uuid: null
            })
            if(!this.state.selected_step_template) {
                this.setState({
                   selected_step_template: _.last(step_templates)
                });
            }
            step_templates.push({
                step_type: "Land",
                name: "Land",
                poi_uuid: null
            })
            for (let poi of this.props.project.map_layers.layer_pois.pois) {
                step_templates.push({
                    step_type: "POI",
                    poi_uuid: poi.uuid,
                    name: `POI: ${poi.name}`,
                })
            }
        }

        console.log('value_delay_before_execution_sec', value_delay_before_execution_sec);

        return (
            <div>
                <Modal
                    open={this.props.isOpen}
                    onClose={this.onClose}
                >
                    <Grid container spacing={0} alignItems="center" direction="column" style={contain_style}>
                        <Grid item sx={title_bar_style}>Flight Plan</Grid>
                        <Grid item sx={style}>
                            <div style={grid_container}>
                                <div>Include in<br/>Loop</div>
                                <div>Step</div>
                                <div>Status</div>
                                <div>Actions</div>
                            </div>
                            <div className="pois_wrapper_style">
                                {this.props.project && this.props.project.flight_plan &&
                                    this.props.project.flight_plan.steps.length > 0 ?
                                <SortableFlightPlanPOIs
                                    project={this.props.project}
                                    onChange={this.handleOnChange}
                                    isFlightPlanExecuting={this.props.isFlightPlanExecuting}
                                    onReOrderSteps={(steps_order_uuids) => {
                                        let steps = [];
                                        for(let step_uuid of steps_order_uuids) {
                                            let step = _.find(this.props.project.flight_plan.steps, {step_uuid: step_uuid});
                                            steps.push(step);
                                        }
                                        this.props.project.flight_plan.steps = steps;
                                        eel.update_flight_plan_dict(this.props.project.flight_plan);
                                        this.setState({});
                                    }}
                                    onDeleteStepByUUID={(step_uuid) => {
                                        if(step_uuid) {
                                            this.props.project.flight_plan.steps = _.filter(this.props.project.flight_plan.steps, (step) => {
                                                return step.step_uuid !== step_uuid;
                                            });
                                            eel.update_flight_plan_dict(this.props.project.flight_plan);
                                            this.setState({});
                                        }
                                    }}
                                    /> :
                                <div className="pois_wrapper_style_no_steps">No steps added</div>}
                            </div>
                            <div className="clear"></div>
                            <div className="pois_loop_box_add_step">
                                <div>
                                    <Select
                                        disabled={this.props.isFlightPlanExecuting}
                                        displayEmpty
                                        style={{
                                            width: "240px",
                                            marginLeft: "10px",
                                        }}
                                        value={this.state.selected_step_template.name}
                                        onChange={(event) => {
                                            this.setState({
                                                selected_step_template: step_templates.find(it => it.name === event.target.value)
                                            });
                                        }}
                                        renderValue={(val) => {
                                            return <>{val}</>
                                        }}
                                    >
                                        {step_templates.map((step_template) => (
                                            <MenuItem
                                                key={step_template.name}
                                                value={step_template.name}>{step_template.name}</MenuItem>
                                        ))}
                                    </Select>
                                </div>
                                <div>
                                    <Button
                                        variant="action"
                                        disabled={this.props.isFlightPlanExecuting}
                                        style={{
                                            width: "240px",
                                            marginLeft: "10px",
                                            marginTop: "10px",
                                        }}
                                        onClick={() => {
                                            let can_add_step = true;
                                            //Get the last step
                                            let steps_list = this.props.project.flight_plan.steps;
                                            let last_step;
                                            if (steps_list.length) {
                                                last_step = steps_list[steps_list.length - 1];
                                            }

                                            // If the step is first one then check if it is take off
                                            if (!last_step && this.state.selected_step_template.step_type !== "TakeOff") {
                                                this.setState({
                                                    isDialogWarning: true,
                                                    message: "First step must be Take Off!",
                                                })
                                                can_add_step = false;
                                            } else if (last_step) {
                                                if (last_step.step_type === "TakeOff" && this.state.selected_step_template.step_type === "Land") {
                                                    // If the landing comes after take off
                                                    this.setState({
                                                        isDialogWarning: true,
                                                        message: "Must select POIs between take off and landing!",
                                                    })
                                                    can_add_step = false;
                                                } else if (last_step.step_type === "POI" && this.state.selected_step_template.step_type === "TakeOff") {
                                                    // If the takeoff comes after poi
                                                    this.setState({
                                                        isDialogWarning: true,
                                                        message: "Cannot take off after POIs. Must land first!",
                                                    })
                                                    can_add_step = false;
                                                } else if (last_step.step_type === "Land" && this.state.selected_step_template.step_type === "POI") {
                                                    // If the poi comes after Land
                                                    this.setState({
                                                        isDialogWarning: true,
                                                        message: "Cannot select POIs after landing. Must take off first!",
                                                    })
                                                    can_add_step = false;
                                                } else if (last_step.step_type === "Land" && this.state.selected_step_template.step_type === "TakeOff") {
                                                    // Allow to take off after land
                                                } else if (this.state.selected_step_template.poi_uuid === last_step.poi_uuid) {
                                                    // If the poi is being added consecutively show warning
                                                    this.setState({
                                                        isDialogWarning: true,
                                                        message: `Cannot add the same ${this.state.selected_step_template.name} consecutively!`,
                                                    })
                                                    can_add_step = false;
                                                }
                                            }

                                            if (can_add_step) {
                                                this.props.project.flight_plan.steps.push({
                                                        step_uuid: (new Date()).getTime().toString(),
                                                        step_type: this.state.selected_step_template.step_type,
                                                        poi_uuid: this.state.selected_step_template.poi_uuid,
                                                        name: this.state.selected_step_template.name,
                                                        step_status: "NotSet",
                                                        is_included_in_loop: !!this.state.selected_step_template.poi_uuid,
                                                    });
                                                    eel.update_flight_plan_dict(this.props.project.flight_plan);
                                                    this.setState({});
                                            }
                                    }}>Add Step</Button>
                                </div>
                            </div>
                            <div className="clear"></div>
                            <div className="pois_loop_box">
                                 <TextField variant="outlined"
                                   defaultValue={value_repeat_count}
                                   value={this.props.isFlightPlanExecuting ? value_repeat_count : undefined}
                                   onChange={(e) => {
                                       try {
                                           let candidate_value = parseInt(e.target.value);
                                           if(candidate_value > 1) {
                                               this.props.project.flight_plan.repeat_count = candidate_value;
                                               eel.update_flight_plan_dict(this.props.project.flight_plan);
                                               this.setState({});
                                           }
                                       } catch (e) {
                                       }
                                   }}
                                   disabled={this.props.isFlightPlanExecuting}
                                   label={"Repeated loop Count"}
                                   sx={{
                                        backgroundColor: '#fff',
                                        border: "1px solid #D5DADC",
                                        borderRadius: "5px",
                                        margin: "0 10px",
                                        width: "240px"
                                    }}/>
                                <TextField variant="outlined"
                                   defaultValue={value_delay_before_execution_sec}
                                   value={this.props.isFlightPlanExecuting ? value_delay_before_execution_sec : undefined}
                                   onChange={(e) => {
                                       try {
                                           this.props.project.flight_plan.delay_before_execution_sec = parseInt(e.target.value);
                                           eel.update_flight_plan_dict(this.props.project.flight_plan);
                                           this.setState({});
                                       } catch (e) {
                                       }
                                   }}
                                   disabled={this.props.isFlightPlanExecuting}
                                   label={"Delay before execute, sec."}
                                   sx={{
                                        backgroundColor: '#fff',
                                        border: "1px solid #D5DADC",
                                        borderRadius: "5px",
                                        margin: "0 10px",
                                        width: "240px"
                                    }}/>
                            </div>
                            <Button variant="action" style={{padding: "4%"}} onClick={this.onClose}>Close</Button>
                            <Button variant="action" style={{padding: "4%"}} onClick={this.onExecute}>
                                {this.props.isFlightPlanExecuting ? "Stop plan" : "Execute plan"}</Button>
                        </Grid>
                    </Grid>
                </Modal>
                <DialogWarning
                    message={this.state.message}
                    isOpen={this.state.isDialogWarning} onClose={() => {
                    this.setState({isDialogWarning: false})
                }}/>
            </div>
        );
    }
}